@@ -3,11 +3,11 @@
 from functools import wraps
 
 from trytond.i18n import gettext
-from trytond.pool import PoolMeta, Pool
 from trytond.model import ModelView, Workflow, fields
 from trytond.model.exceptions import AccessError
-from trytond.transaction import Transaction
+from trytond.pool import Pool, PoolMeta
 from trytond.pyson import Eval
+from trytond.transaction import Transaction
 
 from .exceptions import PurchaseWarehouseWarning
 
@@ -70,7 +70,7 @@ class Purchase(metaclass=PoolMeta):
 
     @classmethod
     @ModelView.button
-    @Workflow.transition('cancel')
+    @Workflow.transition('cancelled')
     @process_request
     def cancel(cls, purchases):
         super(Purchase, cls).cancel(purchases)
@@ -91,6 +91,12 @@ class Line(metaclass=PoolMeta):
             'invisible': ~Eval('requests'),
             })
 
+    @classmethod
+    def copy(cls, lines, default=None):
+        default = default.copy() if default is not None else {}
+        default.setdefault('requests')
+        return super().copy(lines, default=default)
+
     @classmethod
     def delete(cls, lines):
         pool = Pool()
