@@ -2,26 +2,24 @@
 # this repository contains the full copyright notices and license terms.
 import datetime
 from decimal import Decimal
-from itertools import groupby
 from functools import partial
+from itertools import groupby
 
 from trytond.i18n import gettext
-from trytond.model import ModelView, ModelSQL, fields
+from trytond.model import ModelSQL, ModelView, fields
 from trytond.model.exceptions import AccessError
-from trytond.wizard import Wizard, StateView, StateTransition, Button
-from trytond.pyson import If, Eval, Bool
-from trytond.tools import sortable_values, firstline
-from trytond.transaction import Transaction
-from trytond.pool import Pool
-
 from trytond.modules.company.model import (
-    employee_field, set_employee, reset_employee)
+    employee_field, reset_employee, set_employee)
 from trytond.modules.product import round_price
+from trytond.pool import Pool
+from trytond.pyson import Bool, Eval, If
+from trytond.tools import firstline, sortable_values
+from trytond.transaction import Transaction
+from trytond.wizard import Button, StateTransition, StateView, Wizard
 
 STATES = {
     'readonly': Eval('state') != 'draft',
     }
-DEPENDS = ['state']
 
 
 class PurchaseRequest(ModelSQL, ModelView):
@@ -34,18 +32,17 @@ class PurchaseRequest(ModelSQL, ModelView):
         context={
             'company': Eval('company', -1),
             },
-        depends=['company'])
-    description = fields.Text('Description', readonly=True,
-        states=STATES, depends=DEPENDS)
+        depends={'company'})
+    description = fields.Text('Description', readonly=True, states=STATES)
     summary = fields.Function(fields.Char('Summary'), 'on_change_with_summary')
     party = fields.Many2One(
         'party.party', "Party", select=True, states=STATES,
         context={
             'company': Eval('company', -1),
             },
-        depends=DEPENDS + ['company'])
+        depends={'company'})
     quantity = fields.Float('Quantity', required=True, states=STATES,
-        digits=(16, Eval('uom_digits', 2)), depends=DEPENDS + ['uom_digits'])
+        digits=(16, Eval('uom_digits', 2)))
     uom = fields.Many2One('product.uom', 'UOM', select=True,
         ondelete='RESTRICT',
         domain=[
@@ -56,8 +53,7 @@ class PurchaseRequest(ModelSQL, ModelView):
         states={
             'required': Bool(Eval('product')),
             'readonly': STATES['readonly'],
-            },
-        depends=['product', 'product_uom_category'] + DEPENDS)
+            })
     uom_digits = fields.Function(fields.Integer('UOM Digits'),
         'on_change_with_uom_digits')
     product_uom_category = fields.Function(
@@ -73,14 +69,13 @@ class PurchaseRequest(ModelSQL, ModelView):
     default_uom_digits = fields.Function(fields.Integer('Default UOM Digits'),
         'on_change_with_default_uom_digits')
     stock_level = fields.Float('Stock at Supply Date', readonly=True,
-        digits=(16, Eval('default_uom_digits', 2)),
-        depends=['default_uom_digits'])
+        digits=(16, Eval('default_uom_digits', 2)))
     warehouse = fields.Many2One(
         'stock.location', "Warehouse",
         states={
             'required': Eval('warehouse_required', False),
             },
-        domain=[('type', '=', 'warehouse')], depends=['warehouse_required'],
+        domain=[('type', '=', 'warehouse')],
         readonly=True)
     warehouse_required = fields.Function(fields.Boolean('Warehouse Required'),
         'get_warehouse_required')
@@ -96,12 +91,12 @@ class PurchaseRequest(ModelSQL, ModelView):
     purchased_by = employee_field(
         "Purchased By", states=['purchased', 'done', 'cancelled', 'exception'])
     state = fields.Selection([
+            ('draft', "Draft"),
             ('purchased', "Purchased"),
             ('done', "Done"),
-            ('draft', "Draft"),
             ('cancelled', "Cancelled"),
             ('exception', "Exception"),
-            ], "State", required=True, readonly=True, select=True)
+            ], "State", required=True, readonly=True, select=True, sort=False)
 
     @classmethod
     def __setup__(cls):
@@ -199,14 +194,22 @@ class PurchaseRequest(ModelSQL, ModelView):
 
     @classmethod
     def search_rec_name(cls, name, clause):
-        res = []
-        names = clause[2].split('@', 1)
-        res.append(('product.template.name', clause[1], names[0]))
-        if len(names) != 1 and names[1]:
-            res.append(('warehouse', clause[1], names[1]))
-        return ['OR', res,
-            ('description',) + tuple(clause[1:]),
-            ]
+        _, operator, value = clause
+        if operator.startswith('!') or operator.startswith('not'):
+            bool_op = 'AND'
+        else:
+            bool_op = 'OR'
+        domain = [bool_op]
+        if value is not None:
+            names = value.split('@', 1)
+            sub_domain = [('product.template.name', operator, names[0])]
+            if len(names) != 1 and names[1]:
+                sub_domain.append(('warehouse', operator, names[1]))
+                if bool_op == 'AND':
+                    sub_domain.insert(0, 'OR')
+            domain.append(sub_domain)
+        domain.append(('description', *clause[1:]))
+        return domain
 
     @staticmethod
     def default_company():
@@ -226,7 +229,10 @@ class PurchaseRequest(ModelSQL, ModelView):
 
     @property
     def currency(self):
-        return self.company.currency
+        currency = self.company.currency
+        if self.party and self.party.supplier_currency:
+            currency = self.party.supplier_currency
+        return currency
 
     @classmethod
     def default_state(cls):
@@ -342,7 +348,8 @@ class PurchaseRequest(ModelSQL, ModelView):
         "Return the best product supplier to request product at date"
         pool = Pool()
         Date = pool.get('ir.date')
-        today = Date.today()
+        with Transaction().set_context(context=product._context):
+            today = Date.today()
         for product_supplier in product.product_suppliers_used(**pattern):
             if date is None:
                 return product_supplier
@@ -366,7 +373,8 @@ class PurchaseRequest(ModelSQL, ModelView):
             purchase_date = product_supplier.compute_purchase_date(date)
         else:
             supplier = None
-            purchase_date = Date.today()
+            with Transaction().set_context(context=product._context):
+                purchase_date = Date.today()
         return supplier, purchase_date
 
     @classmethod
@@ -469,17 +477,24 @@ class CreatePurchase(Wizard):
         if any(reqs):
             return 'ask_party'
 
-        today = Date.today()
-
         requests = [r for r in requests if not r.purchase_line]
 
+        def _sort_keyfunc(requests, request):
+            return (
+                self._group_purchase_key(requests, request)
+                + self._group_purchase_line_key(request))
+        sort_keyfunc = partial(_sort_keyfunc, requests)
+
         keyfunc = partial(self._group_purchase_key, requests)
-        requests = sorted(requests, key=sortable_values(keyfunc))
+        requests = sorted(requests, key=sortable_values(sort_keyfunc))
 
         purchases = []
         lines = []
         for key, grouped_requests in groupby(requests, key=keyfunc):
             grouped_requests = list(grouped_requests)
+            key = dict(key)
+            with Transaction().set_context(company=key['company']):
+                today = Date.today()
             try:
                 purchase_date = min(r.purchase_date
                     for r in grouped_requests
@@ -489,7 +504,7 @@ class CreatePurchase(Wizard):
             if purchase_date < today:
                 purchase_date = today
             purchase = Purchase(purchase_date=purchase_date)
-            for f, v in key:
+            for f, v in key.items():
                 setattr(purchase, f, v)
             purchases.append(purchase)
             for line_key, line_requests in groupby(
@@ -514,15 +529,27 @@ class CreatePurchase(Wizard):
         line.unit_price = round_price(Decimal(0))
         for f, v in key:
             setattr(line, f, v)
-        line.quantity = sum(r.quantity for r in requests)
         line.purchase = purchase
         line.on_change_product()
+        line.quantity = cls.compute_quantity(requests, line, purchase)
+        if line.unit:
+            line.quantity = line.unit.ceil(line.quantity)
         # Set again in case on_change's changed them
         for f, v in key:
             setattr(line, f, v)
         line.on_change_quantity()
         return line
 
+    @classmethod
+    def compute_quantity(cls, requests, line, purchase):
+        pool = Pool()
+        Uom = pool.get('product.uom')
+        unit = line.unit
+        compute_qty = Uom.compute_qty
+        return sum(
+            compute_qty(r.uom, r.quantity, unit, round=False)
+            for r in requests)
+
 
 class HandlePurchaseCancellationException(Wizard):
     'Handle Purchase Cancellation Exception'
